// ZenLib::Ztring - std::string is better
// Copyright (C) 2002-2004 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// String
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.1.0
// -------------
// Gestion pousses des chaines de caractres
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#ifndef ZenLib_ZtringH
#define ZenLib_ZtringH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <ZenLib/Conf.h>
#include <string>
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#ifdef _UNICODE
    #define ZENLIB_STRINGTYPE std::wstring
    #define ZENLIB_STRINGTYPE_SHORT wstring
#else
    #define ZENLIB_STRINGTYPE std::string
    #define ZENLIB_STRINGTYPE_SHORT string
#endif
//---------------------------------------------------------------------------

namespace ZenLib
{

//---------------------------------------------------------------------------
/// @brief Options for Ztring methods
enum ztring_t
{
    Ztring_Nothing,
    Ztring_Rounded              = 1,            ///< if >.5, upper, else lower
    Ztring_CaseSensitive        = 2,            ///< Case sensitive ("A" and "a" are different)
    Ztring_AddLastItem          = 4,            ///< if Begin is found and End is not found, return between Begin and end of string
    Ztring_Recursive            = 8             ///< Do all strings
};

//---------------------------------------------------------------------------

//***************************************************************************
/// @brief String manipulation (based on std::string)
/// @version 0.0.1
//***************************************************************************

using ZENLIB_STRINGTYPE;
class Ztring : public ZENLIB_STRINGTYPE  //for details about undocumented methods see http://www.sgi.com/tech/stl/basic_string.html
{
public :
    //Gestion de la classe
    Ztring ()                                                                   : ZENLIB_STRINGTYPE_SHORT(){};
    Ztring (const ZENLIB_STRINGTYPE& s, size_type pos=0, size_type n=npos)      : ZENLIB_STRINGTYPE_SHORT(s.c_str(), pos, npos){};
    Ztring (const ZENLIB_STRINGTYPE* s, size_type pos=0, size_type n=npos)      : ZENLIB_STRINGTYPE_SHORT(*s, pos, npos){};
    Ztring (const Char *s)                                                      : ZENLIB_STRINGTYPE_SHORT(s){};
    Ztring (const Char *s, size_type n)                                         : ZENLIB_STRINGTYPE_SHORT(s, n){};
    Ztring (size_type n, const Char s)                                          : ZENLIB_STRINGTYPE_SHORT(n, s){};
    Ztring (const ints);
    Ztring (const intu);
#if (MAXTYPE_INT >= 64 && UINT_MAX == 0xfffffffful) //We are not on 64 bit platform
	Ztring (const int64s);
	Ztring (const int64u);
#endif
    Ztring (const float32);
    Ztring (const float64);


    //Operators
    Ztring operator + (const Ztring &) const;
    Ztring operator + (const Char *) const;
    Ztring &operator += (const Ztring &);
    Ztring &operator += (const Char *);
    Ztring &operator = (const Ztring &);
    Ztring &operator = (const Char *);
    bool operator == (const Ztring &Value) const                                {return compare(Ztring(Value))==0;};
    bool operator == (const Char *Value) const                                  {return compare(Ztring(Value))==0;};
    bool operator != (const Ztring &Value) const                                {return compare(Ztring(Value))!=0;};
    bool operator != (const Char *Value) const                                  {return compare(Ztring(Value))!=0;};

    //Conversions
    static Ztring ToZtring (const int8s, int radix=10);
    static Ztring ToZtring (const int8u, int radix=10);
    static Ztring ToZtring (const int16s, int radix=10);
    static Ztring ToZtring (const int16u, int radix=10);
    static Ztring ToZtring (const int32s, int radix=10);
    static Ztring ToZtring (const int32u, int radix=10);
#if (MAXTYPE_INT >= 64)
    static Ztring ToZtring (const int64s, int radix=10);
    static Ztring ToZtring (const int64u, int radix=10);
#endif
    static Ztring ToZtring (const float32, int decimal=3);
    static Ztring ToZtring (const float64, int decimal=3);
        /// @brief return a new char array
        /// @warning you MUST delete this pointer! (delete[])
    Char* Convert_Char() const;
        /// @brief Convert into Int (32 bits)
        /// @return the value corresponding \n
        ///         0 if there is a problem
    int32s Convert_int32s(ztring_t Options=Ztring_Rounded) const;
        /// @brief Convert into unsigned Int (32 bits)
        /// @return the value corresponding
        ///         0 if there is a problem
    int32u Convert_int32u(ztring_t Options=Ztring_Rounded) const;
#if (MAXTYPE_INT >= 64)
        /// @brief Convert into Int (64 bits)
        /// @return the value corresponding \n
        ///         0 if there is a problem
    int64s Convert_int64s(ztring_t Options=Ztring_Rounded) const;
        /// @brief Convert into unsigned Int (64 bits)
        /// @return the value corresponding \n
        ///         0 if there is a problem
    int64u Convert_int64u(ztring_t Options=Ztring_Rounded) const;
#endif
        /// @brief Convert into float
        /// @return the value corresponding \n
        ///         0 if there is a problem
    float32 Convert_float32(ztring_t Options=Ztring_Rounded) const; //Vers decimal
    float64 Convert_float64(ztring_t Options=Ztring_Rounded) const; //Vers decimal

    //Edition
        /// @brief convert into lowercase
    Ztring* MakeLowerCase();
        /// @brief convert into uppercase
    Ztring* MakeUpperCase();
        /// @brief Remove leading whitespaces from a string
    Ztring* TrimLeft();
        /// @brief Remove trailing whitespaces from a string
    Ztring* TrimRight();
        /// @brief Remove leading and trailing whitespaces from a string
    Ztring* Trim();
        /// @brief Quotes a string
    Ztring* Quote();
        /// @brief return a string between two strings
        /// @param Begin First string
        /// @param End Second string
        /// @param Pos Position to begin to scan string
        /// @param Options Options for searching \n
        ///                Available : Ztring_CaseSensitive
        /// @return The substring \n
        ///         "" if not found
    Ztring SubString (const ZENLIB_STRINGTYPE &Begin, const ZENLIB_STRINGTYPE &End, size_type Pos=0, ztring_t Options=Ztring_Nothing) const;
        /// @brief replace a string by another one
        /// @param ToFind string to find
        /// @param ToReplace string wich replace the string found
        /// @param Pos Position to begin to scan string
        /// @param Options Options for searching \n
        ///                Available : Ztring_CaseSensitive, Ztring_Recursive
        /// @return The substring \n
        ///         "" if not found
    int FindAndReplace (const ZENLIB_STRINGTYPE &ToFind, const ZENLIB_STRINGTYPE &ReplaceBy, size_type Pos=0, ztring_t Options=Ztring_Nothing); //Remplace une chaine par une autre

/*

    //Edition
    int Rechercher (const Ztring &Trouver, int Pos_Debut=0, int Pos_Fin=-1, int Options=0) const; //Retourne la position d'une chaine de caractre. Sens inverse possible, -1 si jusqu'a la fin
    Ztring RechercherEntre (const Ztring &Debut, const Ztring &Fin, int Pos_Debut=0, int Pos_Fin=-1, int Options=0) const; //Retourne une Chaine situe entre deux chaines
    Ztring SousChaine (int Pos_Debut, int Nb_Char=-1) const; //Retourne une Chaine situe entre deux positions (pos debut puis nombre de characteres)
    Ztring SousChaine_Pos (int Pos_Debut, int Pos_Fin=-1) const; //Retourne une Chaine situe entre deux positions (pos debut et pos fin)
    int Remplacer (const Ztring &Trouver, const Ztring &RemplacerPar, int Pos_Debut=0, int Pos_Fin=-1, int Options=0); //Remplace une chaine par une autre
    Ztring Reformater_Sortie (const Ztring &Trouver, const Ztring &RemplacerPar, int Pos_Debut=0, int Pos_Fin=-1, int Options=0); //Remplace une chaine par une autre
    int Compter(const Ztring &ACompter, int Options=0) const; //Compte le nombre d'occurence d'une chaine
    bool Comparer (const Ztring &AComparer, const Ztring &Comparateur, int Options=0) const; //Comapre une chaine avec une autre
*/
};

} //NameSpace
#endif


