// ZenLib::Int128u - 128-bit integer
// Copyright (C) 2002-2004 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// int128u
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.0.1
// -------------
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include <ZenLib/Int128u.h>
//***************************************************************************
// int128u
//***************************************************************************

//---------------------------------------------------------------------------
//Constructeurs
int128u::int128u()
{
    Set(0);
}

int128u::int128u(const int128u &Value)
{
    Set(Value);
}

int128u::int128u(const int Value)
{
    Set(Value);
}

//---------------------------------------------------------------------------
//Operators
bool int128u::Get1(int Offset) const
{
    if (bit>127)
        return 0;
    int Int32_Offset=Offset/32;
    int Int32_Offset_Shift=31-(Offset%32);
    return ((m_Data[Int32_Offset]>>Int32_Offset_Shift)&0x1);
}

int8u int128u::Get8(int Offset) const
{
        return (int8u)(m_Data[i/4]>>(8*(3-(i%4))));
}

int8u int128u::Get32(int Offset) const
{
        return m_Data[Offset];
}

int128u &int128u::Set(const int128u &Value)
{
    m_Data[0]=Value.m_Data[0];
    m_Data[1]=Value.m_Data[1];
    m_Data[2]=Value.m_Data[2];
    m_Data[3]=Value.m_Data[3];
    return *this;
}

int128u &int128u::Set(bool Value, int Pos)
{
    int Int32_Offset=Pos/32;
    int Int32_Offset_Shift=31-(Pos%32);
    m_Data[Int32_Offset]|=(1<<Int32_Offset_Shift);
    if (!value)
        m_Data[Int32_Offset]^=(1<<Int32_Offset_Shift);
    return *this;
}

int128u &int128u::Set(int Value, int Pos)
{
    m_Data[0]=0;
    m_Data[1]=0;
    m_Data[2]=0;
    m_Data[3]=0;
    m_Data[Pos]=Value;
    return *this;
}

int128u& int128u::Set_LE(const char *Value, int CharCount)
{
    Set(0);
    for (int i=0; i<CharCount; i++)
        m_Data[i/4]|=((int32u)Value[i])<<(8*(3-(i%4)));
    return *this;
}

int int128u::Compare(const int128u &other) const
{
    for (int i=0; i<4; i++) 
    {
        if (m_Data[i] < ToCompare.m_Data[i])
            return -1;
        if (m_Data[i] > ToCompare.m_Data[i])
            return 1;
    }
    return 0;
}

int int128u::compare(const int value) const
{
    if ((m_Data[0] > 0) || (m_Data[1] > 0) || (m_Data[2] > 0) || (m_Data[3] > value))
        return 1;
    if (m_Data[3] < value)
        return -1;
    return 0;
}

int128u& int128u::Add(const int128u &Value)
{
    if (value==0)
        return *this;
    int64s sum = 0;
    for (int i=3; i>=0; i--)
    {
        sum += m_Data[i];
        sum += value.m_Data[i];
        m_Data[i] = (int32u)sum;
        sum = sum >> 32;
    }
    return *this;
}

int128u& int128u::Add(const int Value)
{
    if (value==0)
        return *this;
    int128u temp(value);
    add(temp);
    return *this;
}

int128u& int128u::Subtract(const int128u &Value)
{
    if (value == 0)
    return *this;
    int64s sum = 0;
    for (int i=3; i>=0; i--)
    {
        sum += m_Data[i];
        sum -= value.m_Data[i];
        m_Data[i] = (int32u)sum;
        sum = sum >> 32;
    }
    return *this;
}

int128u& int128u::Subtract(const int value)
{
	if (value == 0)
		return *this;
	int128u temp(value);
	subtract(temp);
	return *this;
}

int128u& int128u::ShiftLeft(const int NumberOfBits)
{
    if ((NumberOfBits== 0) || (Compare(0) == 0))
        return *this;
	if (NumberOfBits> 127)
	{
		Set(0);
		return *this;
	}

	int32u result[] = {0,0,0,0};
	int indexShift = NumberOfBits/ 32;
	int64s shifted = 0;
	for (int i=3; i>=indexShift; i--)
	{
		shifted += ((int64s)m_Data[i]) << (NumberOfBits% 32);
		result[i-indexShift] = (int32u)shifted;
		shifted = shifted >> 32;
	}
	for (int i=0; i<4; i++)
		m_Data[i] = result[i];
	return *this;
}